﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace PHPScan.LexicalAnalysis
{
    public struct Token
    {
        public TokenType TokenType;
        public string Lexeme;
        public int Index;

        public Token(TokenType tokenType, string lexeme, int index)
        {
            TokenType = tokenType;
            Lexeme = lexeme;
            Index = index;
        }

        public Token(TokenType tokenType, int index)
            : this(tokenType, null, index)
        {
        }
    }

    partial class LexicalAnalyzer
    {
        private string text = "";

        public string Text
        {
            get { return text; }
            set
            {
                text = value;
                lastIndex = text.Length - 1;
            }
        }

        int lastIndex = 0;

        int charIndex = -1;

        int mode = 0;

        char currentChar = (char)0;

        bool nextCharSucceeded = false;

        public LexicalAnalyzer()
        {
        }

        public LexicalAnalyzer(string text)
        {
            Text = text;
        }

        bool NextChar()
        {
            if (charIndex < lastIndex)
            {
                currentChar = text[++charIndex];
                return true;
            }

            return false;
        }

        void PreviousChar()
        {
            //currentChar = text[--charIndex];
            charIndex--;
        }

        public TokenType GetToken()
        {

            if (mode == 0)
            {
                if (charIndex < lastIndex)
                {
                    currentChar = text[++charIndex];

                    switch (currentChar)
                    {
                        case '<':
                            if (charIndex < lastIndex)
                            {
                                currentChar = text[++charIndex];

                                switch (currentChar)
                                {
                                    case '?':
                                        if (charIndex < lastIndex)
                                        {
                                            currentChar = text[++charIndex];

                                            switch (currentChar)
                                            {
                                                case 'p':
                                                    if (charIndex < lastIndex)
                                                    {
                                                        currentChar = text[++charIndex];

                                                        switch (currentChar)
                                                        {
                                                            case 'h':
                                                                if (charIndex < lastIndex)
                                                                {
                                                                    currentChar = text[++charIndex];

                                                                    switch (currentChar)
                                                                    {
                                                                        case 'p':

                                                                            mode = 1;
                                                                            return TokenType.OpenTag;

                                                                    }

                                                                    charIndex--;
                                                                }
                                                                return TokenType.Unknown;

                                                        }

                                                        charIndex--;
                                                    }
                                                    return TokenType.Unknown;

                                                case '=':

                                                    mode = 1;
                                                    return TokenType.OpenWithEcho;

                                            }

                                            charIndex--;
                                        }
                                        mode = 1;
                                        return TokenType.OpenTag;

                                    case '%':
                                        if (charIndex < lastIndex)
                                        {
                                            currentChar = text[++charIndex];

                                            switch (currentChar)
                                            {
                                                case '=':

                                                    mode = 1;
                                                    return TokenType.OpenWithEcho;

                                            }

                                            charIndex--;
                                        }
                                        mode = 1;
                                        return TokenType.OpenTag;

                                }

                                charIndex--;
                            }
                            return TokenType.Unknown;

                        default:

                            var state = 0;

                            do
                            {
                                if (state == 0 && currentChar == '<')
                                    state = 1;
                                else if (state == 1 && currentChar == '?')
                                {
                                    charIndex -= 2;
                                    return TokenType.Html;
                                }
                                else
                                    state = 0;
                            }
                            while (NextChar());

                            return TokenType.Html;
                            break;

                    }

                    return TokenType.Unknown;
                }
            }
            else
                if (mode == 1)
                {
                    if (charIndex < lastIndex)
                    {
                        currentChar = text[++charIndex];

                        switch (currentChar)
                        {
                            case '&':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '&':

                                            return TokenType.AndOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.Ampersand;

                            case ',':

                                return TokenType.Comma;

                            case ':':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case ':':

                                            return TokenType.DoubleColon;

                                    }

                                    charIndex--;
                                }
                                return TokenType.TernaryOperator;

                            case '\\':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '\\':

                                            return TokenType.Namespace;

                                    }

                                    charIndex--;
                                }
                                return TokenType.Unknown;

                            case '@':

                                return TokenType.ErrorSuppressor;

                            case '(':

                                return TokenType.LeftParenthesis;

                            case ')':

                                return TokenType.RightParenthesis;

                            case '[':

                                return TokenType.LeftBracket;

                            case ']':

                                return TokenType.RightBracket;

                            case '{':

                                return TokenType.LeftBrace;

                            case '}':

                                return TokenType.RightBrace;

                            case '=':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':
                                            if (charIndex < lastIndex)
                                            {
                                                currentChar = text[++charIndex];

                                                switch (currentChar)
                                                {
                                                    case '=':

                                                        return TokenType.IdenticalOperator;

                                                }

                                                charIndex--;
                                            }
                                            return TokenType.EqualityOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.AssignmentOperator;

                            case '+':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.PlusEqualOperator;

                                        case '+':

                                            return TokenType.IncrementOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.AdditionOperator;

                            case '-':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.MinusEqualOperator;

                                        case '-':

                                            return TokenType.DecrementOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.SubtractionOperator;

                            case '*':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.MultiplicationEqualOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.MultiplicationOperator;

                            case '/':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.DivisionEqualOperator;

                                        case '/':


                                            while (NextChar())
                                            {
                                                if (currentChar == '\r' || currentChar == '\n')
                                                {
                                                    charIndex--;

                                                    return TokenType.Comment;
                                                }
                                            }

                                            return TokenType.Comment;
                                            break;

                                        case '*':


                                            var state = 0;

                                            while (NextChar())
                                            {
                                                if (state == 0 && currentChar == '*')
                                                    state = 1;
                                                else if (state == 1 && currentChar == '/')
                                                    return TokenType.Comment;
                                                else
                                                    state = 0;
                                            }

                                            return TokenType.Comment;
                                            break;

                                    }

                                    charIndex--;
                                }
                                return TokenType.DivisionOperator;

                            case '%':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.ModulusEqualOperator;

                                        case '>':

                                            mode = 0;
                                            return TokenType.CloseTag;

                                    }

                                    charIndex--;
                                }
                                return TokenType.ModulusOperator;

                            case '.':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.ConcatEqualOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.StringConcatOperator;

                            case '|':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.OrEqualOperator;

                                        case '|':

                                            return TokenType.OrOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.OrOperator;

                            case '^':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.XorEqualOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.XorOperator;

                            case '!':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':
                                            if (charIndex < lastIndex)
                                            {
                                                currentChar = text[++charIndex];

                                                switch (currentChar)
                                                {
                                                    case '=':

                                                        return TokenType.NotIdenticalOperator;

                                                }

                                                charIndex--;
                                            }
                                            return TokenType.NotEqualOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.NotOperator;

                            case '<':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '>':

                                            return TokenType.NotEqualOperator;

                                        case '=':

                                            return TokenType.LessThanOrEqualOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.LessThanOperator;

                            case '>':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '=':

                                            return TokenType.GreaterThanOrEqualOperator;

                                    }

                                    charIndex--;
                                }
                                return TokenType.GreaterThanOperator;

                            case '?':
                                if (charIndex < lastIndex)
                                {
                                    currentChar = text[++charIndex];

                                    switch (currentChar)
                                    {
                                        case '>':

                                            mode = 0;
                                            return TokenType.CloseTag;

                                    }

                                    charIndex--;
                                }
                                return TokenType.TernaryOperator;

                            case ';':

                                return TokenType.EndOfStatement;

                            case '\x0d':

                                return TokenType.WhiteSpace;

                            case '\x0a':

                                return TokenType.WhiteSpace;

                            case '\x09':

                                return TokenType.WhiteSpace;

                            case '\x0b':

                                return TokenType.WhiteSpace;

                            case ' ':

                                return TokenType.WhiteSpace;

                            case '$':


                                var variablestate = 0;

                                while (NextChar())
                                {
                                    if (variablestate == 0 &&
                                        ((currentChar >= 'a' && currentChar <= 'z') ||
                                        (currentChar >= 'A' && currentChar <= 'Z') ||
                                        currentChar == '_' ||
                                        (currentChar >= '\x7f' && currentChar <= '\xff')))
                                        variablestate = 1;
                                    else if (variablestate == 1 &&
                                        ((currentChar >= 'a' && currentChar <= 'z') ||
                                        (currentChar >= 'A' && currentChar <= 'Z') ||
                                        (currentChar >= '0' && currentChar <= '9') ||
                                        currentChar == '_' ||
                                        (currentChar >= '\x7f' && currentChar <= '\xff')))
                                        variablestate = 1;
                                    else if (variablestate == 1 || variablestate == 2)
                                    {
                                        charIndex--;

                                        return TokenType.Variable;
                                    }
                                    else
                                    {
                                        break;
                                    }
                                }

                                break;

                            case '"':


                                bool escaped = false;

                                while (NextChar())
                                {
                                    if (!escaped && currentChar == '"')
                                        return TokenType.String;

                                    escaped = !escaped && currentChar == '\\';
                                }

                                return TokenType.String;

                                break;

                            case '\'':


                                escaped = false;

                                while (NextChar())
                                {
                                    if (!escaped && currentChar == '\'')
                                        return TokenType.String;

                                    escaped = !escaped && currentChar == '\\';
                                }

                                return TokenType.String;

                                break;

                            case '#':


                                while (NextChar())
                                {
                                    if (currentChar == '\r' || currentChar == '\n')
                                    {
                                        charIndex--;

                                        return TokenType.Comment;
                                    }
                                }

                                return TokenType.Comment;
                                break;

                            default:

                                var identifierState = 0;

                                do
                                {
                                    if (identifierState == 0 &&
                                        ((currentChar >= 'a' && currentChar <= 'z') ||
                                        (currentChar >= 'A' && currentChar <= 'Z') ||
                                        currentChar == '_' ||
                                        (currentChar >= '\x7f' && currentChar <= '\xff')))
                                        identifierState = 1;
                                    else if (identifierState == 1 &&
                                        ((currentChar >= 'a' && currentChar <= 'z') ||
                                        (currentChar >= 'A' && currentChar <= 'Z') ||
                                        (currentChar >= '0' && currentChar <= '9') ||
                                        currentChar == '_' ||
                                        (currentChar >= '\x7f' && currentChar <= '\xff')))
                                        identifierState = 1;
                                    else if (identifierState == 1 || identifierState == 2)
                                    {
                                        charIndex--;

                                        return TokenType.Identifier;
                                    }
                                    else
                                    {
                                        break;
                                    }
                                }
                                while (NextChar());


                                var numberState = 0;

                                do
                                {
                                    if ((numberState == 0 || numberState == 1) && currentChar > 47 && currentChar < 58)
                                        numberState = 1;
                                    else if (numberState == 1 && currentChar == '.')
                                        numberState = 2;
                                    else if (numberState == 2 && currentChar > 47 && currentChar < 58)
                                        numberState = 3;
                                    else if (numberState == 1 || numberState == 3)
                                    {
                                        charIndex--;

                                        return TokenType.Number;
                                    }
                                    else
                                    {
                                        break;
                                    }
                                }
                                while (NextChar());

                                break;

                        }

                        return TokenType.Unknown;
                    }
                }


            return TokenType.EndOfFile;
        }

        public List<Token> GetTokens()
        {
            charIndex = -1;

            TokenType tokenType;

            int lastIndex = -1;

            var tokens = new List<Token>();

            while ((tokenType = GetToken()) != TokenType.EndOfFile)
            {
                var index = lastIndex + 1;
                var lexeme = text.Substring(index, charIndex - lastIndex);

                switch (lexeme.Length)
                {
                    case 8:
                        if (lexeme == "abstract" ||
                        lexeme == "continue" ||
                        lexeme == "endwhile" ||
                        lexeme == "function")
                            tokenType = TokenType.Keyword;
                        break;
                    case 3:
                        if (lexeme == "and" ||
                        lexeme == "for" ||
                        lexeme == "new" ||
                        lexeme == "try" ||
                        lexeme == "use" ||
                        lexeme == "var" ||
                        lexeme == "xor")
                            tokenType = TokenType.Keyword;
                        break;
                    case 5:
                        if (lexeme == "array" ||
                        lexeme == "break" ||
                        lexeme == "catch" ||
                        lexeme == "class" ||
                        lexeme == "clone" ||
                        lexeme == "const" ||
                        lexeme == "endif" ||
                        lexeme == "final" ||
                        lexeme == "throw" ||
                        lexeme == "while")
                            tokenType = TokenType.Keyword;
                        break;
                    case 2:
                        if (lexeme == "as" ||
                        lexeme == "do" ||
                        lexeme == "if" ||
                        lexeme == "or")
                            tokenType = TokenType.Keyword;
                        break;
                    case 4:
                        if (lexeme == "case" ||
                        lexeme == "else" ||
                        lexeme == "goto")
                            tokenType = TokenType.Keyword;
                        break;
                    case 9:
                        if (lexeme == "cfunction" ||
                        lexeme == "endswitch" ||
                        lexeme == "interface" ||
                        lexeme == "namespace" ||
                        lexeme == "protected")
                            tokenType = TokenType.Keyword;
                        break;
                    case 7:
                        if (lexeme == "declare" ||
                        lexeme == "default" ||
                        lexeme == "extends" ||
                        lexeme == "foreach" ||
                        lexeme == "private")
                            tokenType = TokenType.Keyword;
                        break;
                    case 6:
                        if (lexeme == "elseif" ||
                        lexeme == "endfor" ||
                        lexeme == "global" ||
                        lexeme == "public" ||
                        lexeme == "static" ||
                        lexeme == "switch")
                            tokenType = TokenType.Keyword;
                        break;
                    case 10:
                        if (lexeme == "enddeclare" ||
                        lexeme == "endforeach" ||
                        lexeme == "implements" ||
                        lexeme == "instanceof")
                            tokenType = TokenType.Keyword;
                        break;
                    case 12:
                        if (lexeme == "old_function")
                            tokenType = TokenType.Keyword;
                        break;
                }


                tokens.Add(new Token(tokenType, lexeme, index));

                //if (tokenType == TokenType.Unknown)
                //{
                //    Console.Write(lexeme);
                //}

                lastIndex = charIndex;
                //if (token != TokenType.WhiteSpace &&
                //    token != TokenType.Unknown)
                //    Console.WriteLine(token);
            }

            return tokens;
        }
    }
}
