﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.IO;
using PHPScan.LexicalAnalysis;

namespace PHPScan
{
    public class ScanMetrics
    {
        public static ScanMetrics Default = new ScanMetrics();

        public FileInfo AnnotationFile { get; set; }

        private Dictionary<string, int> _annotationTable = new Dictionary<string, int>();

        public Dictionary<string, int> AnnotationTable
        {
            get { return _annotationTable; }
        }

        public string AnnotateCode(string File, string Code)
        {
            var x = 0;
            var matches = Regex
                .Matches(Code, @"/fern\.php'\);")
                .OfType<Match>()
                .Concat(Regex
                    .Matches(Code, @"function\s+" + PHP.ValidNameRegex + @"\([^)]*\)\s*{")
                    .OfType<Match>())
                .Reverse();

            foreach (var match in matches)
            {
                Code = Code.Insert(match.Index + match.Length,
                    string.Format("\r\nAnnotation(\"{0}_{1}\");\r\n",
                        File.Replace("\\", "\\\\"), x++));
            }

            _annotationTable.Add(File, x);

            return Code;
        }

        public string AnnotateFirstOpenTag(string File, string Code)
        {
            var x = _annotationTable[File]++;

            var match = Regex.Match(Code, @"/fern\.php'\);");
            
            Code = Code.Insert(match.Index + match.Length,
                string.Format("\r\nAnnotation(\"{0}_{1}\");\r\n",
                    File.Replace("\\", "\\\\"), x));

            return Code;
        }

        public string AnnotateBlock(string File, string Code, Token[] Tokens, 
            int TokenIndex, bool FunctionsOnly)
        {
            if (Tokens[TokenIndex - 1].TokenType != TokenType.RightParenthesis &&
                Tokens[TokenIndex - 1].TokenType != TokenType.Keyword)
                return Code;

            for (int i = TokenIndex; i >= 0; i--)
            {
                if (Tokens[i].TokenType == TokenType.Keyword)
                {
                    if (Tokens[i].Lexeme == "switch" || 
                        (FunctionsOnly && Tokens[i].Lexeme != "function"))
                        return Code;

                    break;
                }
            }

            var x = _annotationTable[File]++;

            Code = Code.Insert(Tokens[TokenIndex].Index + 1,
                string.Format("\r\nAnnotation(\"{0}_{1}\");\r\n",
                    File.Replace("\\", "\\\\"), x));

            return Code;
        }

        public void Reset()
        {
            AnnotationFile.Delete();
        }

        public CodeCoverageTable CalculateCoverage()
        {
            if (!AnnotationFile.Exists)
                return new CodeCoverageTable();

            var a = File.ReadAllText(AnnotationFile.FullName);

            var entries = a
                .Split(new [] { '\n' }, StringSplitOptions.RemoveEmptyEntries)
                .Distinct();

            var entryTable = new CodeCoverageTable();

            foreach (var entry in entries)
            {
                try
                {
                    var e = entry.RemoveAtLastIndexOf('_');

                    if (entryTable.ContainsKey(e))
                        entryTable[e]++;
                    else
                        entryTable.Add(e, 1);
                }
                catch (System.Exception ex) 
                {
#if DEBUG
                    Console.WriteLine(ex);
#endif
                }                
            }

            var total = 
                entryTable.Sum(x => x.Value) / AnnotationTable.Sum(x => x.Value) * 100;

            foreach (var key in AnnotationTable.Keys)
            {
                if (entryTable.ContainsKey(key))
                    entryTable[key] = entryTable[key] / AnnotationTable[key] * 100;
                else
                    entryTable.Add(key, AnnotationTable[key] == 0 ? 100 : 0);
            }

            entryTable.Add("Total", total);

            return entryTable;
        }
    }
}
