﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Text.RegularExpressions;

namespace PHPScan.ScanPlugins
{
    class XSSScanPlugin : ScanPluginBase
    {
        private string[] _badChars = new string[]
        {
            "<ScRiPt>alert(0)</ScRiPt>",
            "<ScR<script>iPt>alert(0)</ScR<script>iPt>",
            "\"OnMoUsEoVeR=\"alert(0)",
            "\"OnBlUr=\"alert(0)"
        };

        public override int ModeCount
        {
            get { return _badChars.Length * 2; }
        }

        private string _server;

        public override string Server
        {
            get { return _server; }
            set { _server = value; }
        }

        public XSSScanPlugin(string Server)
        {
            _server = Server;
        }

        public override string BuildRequest(int Mode, string TargetFile, FileTrace SourceTrace)
        {
            string chars;

            if (Mode < _badChars.Length)
            {
                chars = _badChars[Mode];
            }
            else
            {
                chars = "\x00" + _badChars[Mode - (ModeCount / 2)];
            }
            
            //return SourceTrace.Calls.CreateRequest(TargetFile, Server,
            //    chars, true, true);
            return SourceTrace.Calls.CreateRequest(TargetFile, Server,
                chars, false, true, true);
        }

        private bool HasAttributeVulnerability(string Response)
        {
            for (int x = 2; x < 4; x++)
            {
                var index = -1;

                var charsLowered = _badChars[x].ToLower();

                while ((index = Response.IndexOf(charsLowered, index + 1)) != -1)
                {


                    for (int i = index; i >= index - 100 && i >= 0; i--)
                    {
                        if (Response[i] == '<')
                            return true;
                        else if (Response[i] == '>')
                            break;
                    }
                }
            }

            return false;
        }

        public override ScanAlert ScanTrace(FileTrace TargetTrace)
        {
            var respLowered = TargetTrace.Response.ToLower();

            if ((respLowered.Contains(_badChars[0].ToLower()) ||
                HasAttributeVulnerability(respLowered)) &&
                Regex.IsMatch(respLowered, @"^http/\d\.\d\s200\s"))
                return new ScanAlert(ScanAlertOptions.Vulnerability,
                    "Reflected XSS", TargetTrace);            
            else
                return null;
        }

        public override string ToString()
        {
            return "Reflected Cross-site Scripting Scan";
        }
    }
}
