﻿using System;
using System.IO;
using System.Reflection;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using PHPScan.ScanPlugins;

namespace PHPScan
{
    public class UI
    {
        public static void SetColors()
        {
            Console.ForegroundColor = ConsoleColor.White;
        }

        public static void DisplayAppInfo()
        {
            string appName = "PHP Vulnerability Hunter";

            Console.WriteLine("{0} {1}\r\n\r\n" +
                "Copyright (C) {2} John Leitch http://www.autosectools.com\r\n",
                appName,
                Assembly.GetExecutingAssembly().GetName().Version.ToString(),
                DateTime.Now.Year);
        }

        public static void DisplayInstructions()
        {
            Console.WriteLine(
@"== Before You Start

For PHP Vulnerability Hunter to successfully run the following 
conditions must be met: 

1) PHP Vulnerability Hunter must be run as administrator 

2) The targeted web application must not be accessed while PHP 
Vulnerability Hunter is running 

3) Only one instance of PHP Vulnerability Hunter per webroot can
be running at any time 

                 
== Arguments

phpvh [-s] [-p] [-m] [-t] [-c] [-c2] [-d] [-v] [-b] [-h] [-r]
    [-dump] webroot apps

webroot       Absolute web root (required)

apps          Application paths (comma delimited, required)
              Use * to scan every application in the webroot

-s            Server (default localhost)

-p            Port (default 80)

-m            Scan modes (default CFLPSXRI)
              C - Arbitrary Command Execution 
              F - Arbitrary File Write/Change/Rename/Delete 
              L - Local File Inclusion/Arbitrary File Read 
              P - Arbitrary PHP Execution 
              S - SQL Injection 
              D - Dynamic Function Call/Class Instantiation
              X - Reflected Cross-site Scripting (XSS)
              R - Open Redirect
              I - Full Path Disclosure

-t timeout    Timeout in milliseconds (default 60000)

-c            Code coverage report

-c2           High accuracy code coverage report 
              WARNING: may cause slowdown and timeouts 

-d            Scan overview report

-v            Open vulnerability report viewer

-b            Beep on vulnerability alert

-h            Hook superglobals
              Alternative to the superglobal override class
              Use to test different code paths or if 
              superglobal override causes errors, although  
              hooking frequently causes errors itself

-r            Repair mode
              Use to repair an application if PHP 
              Vulnerability Hunter crashes during a scan

-dump         Dump all http messages

== Examples

phpvh c:\xampp\htdocs MyApp 
Runs all scans on app located at c:\xampp\htdocs\MyApp

phpvh -m CX c:\xampp\htdocs MyApp1,MyApp2 
Runs Command Execution and Reflected XSS scans on applications 
located at c:\xampp\htdocs\MyApp1 and c:\xampp\htdocs\MyApp2

phpvh c:\xampp\htdocs * 
Runs all scans on every folder in the webroot");


        }

        public static void DisplayCriticalMessage(string Message, params object[] Arguments)
        {
            Console.Write(Message, Arguments);

            Console.WriteLine("\r\n\r\nPress any key to continue...");
            Console.ReadKey();
            Console.WriteLine();
        }

        public static void DisplayCriticalMessageAndExit(string Message, params object[] Arguments)
        {
            DisplayCriticalMessage(Message, Arguments);

            Environment.Exit(0);
        }

        public static void DisplayScanPlugin(ScanPluginBase Plugin)
        {
            Console.ForegroundColor = ConsoleColor.Cyan;
            Console.WriteLine("\r\n" + Plugin.ToString());
            Console.ForegroundColor = ConsoleColor.White;
        }

        public static void DisplayResourcePath(string Path)
        {
            Console.ForegroundColor = ConsoleColor.White;
            Console.WriteLine(Path);
        }

        public static void DisplayResponse(HttpResponse Response, 
            int Mode, int InputCount)
        {
            Console.ForegroundColor = ConsoleColor.Gray;
            Console.WriteLine("{{Mode: {0}, Input Count: {1}}} -> {2}",
                Mode, InputCount, 
                !string.IsNullOrEmpty(Response.Header) ?
                    Response.Header.Split('\r', '\n')[0] :
                    "[No Header]");
        }

        public static void DisplayResponseError(string Response)
        {
            Console.WriteLine("Error reading response");
            //Console.WriteLine(Response);
        }



        public static void DisplayAlert(ScanAlert Alert)
        {
            //Alert.Trace.Request
            Console.ForegroundColor =
                Alert.AlertType == ScanAlertOptions.Warning ?
                ConsoleColor.Yellow : ConsoleColor.Red;
            
            Console.WriteLine(Alert.ToString());

            Console.ForegroundColor = ConsoleColor.White;
        }

        public static void DisplayError(string Error)
        {
            Console.ForegroundColor = ConsoleColor.Red;

            Console.WriteLine(Error);

            Console.ForegroundColor = ConsoleColor.White;
        }

        public static void DisplayAppPath(string Path)
        {
            Console.ForegroundColor = ConsoleColor.Cyan;
            Console.WriteLine("Scanning " + Path);
            Console.ForegroundColor = ConsoleColor.White;
        }

        public static void Pause()
        {
            Console.WriteLine("\r\nPress any key to continue...");
            Console.ReadKey();
        }

        public static void RunAssistant()
        {
            var file = @".\hr";

            var showDialog = 
                !Environment.GetCommandLineArgs().Any(x => x == "-l") &&
                !File.Exists(file);

            File.Create(file).Close();

            if (showDialog)
            {
                if (MessageBox.Show(@"It seems this is the first time PHP " +
                    "Vulnerability Hunter has been run on this system. " + 
                    "Would you like to use the launcher instead of the command line " + 
                    "interface?", "Run Launcher",
                    MessageBoxButtons.YesNo) == DialogResult.Yes)
                {
                    Process.Start("phpvh-gui.exe");
                    Environment.Exit(0);
                }
            }

            
        }
    }
}
