﻿using System;
using System.Linq;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Xml.Serialization;
using System.Diagnostics;
using PHPScan;
using System.Threading;
using System.Reflection;

namespace PHPVHGUI
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        string _configFile = new FileInfo(System.Reflection.Assembly.GetExecutingAssembly().Location).DirectoryName + 
            "\\config.xml";

        string _phpvhExe;

        public ScanConfig Config
        {
            get { return DataContext as ScanConfig; }
        }

        private System.Windows.Forms.FolderBrowserDialog _folderDialog = new System.Windows.Forms.FolderBrowserDialog();

        public Window1()
        {
            InitializeComponent();

            DataContext = new ScanConfig() { RunViewer = true };            

            Loaded += new RoutedEventHandler(Window1_Loaded);
            Closing += new CancelEventHandler(Window1_Closing);

            var l = Assembly.GetExecutingAssembly().Location;

            _phpvhExe = new FileInfo(l).Directory + "\\phpvh.exe";            

        }

        private void SaveConfig()
        {
            Config.Serialize(_configFile);
        }

        private IEnumerable<DirectoryInfo> SearchForPHPDirs(DirectoryInfo TargetDirectory)
        {
            return TargetDirectory
                .GetDirectories()
                .Where(x =>
                {
                    try
                    {
                        return x.GetFiles("*.php").Any();
                    }
                    catch (UnauthorizedAccessException)
                    {
                        return false;
                    }
                });
            //foreach (var subDir in TargetDirectory.GetDirectories())
            //{

            //}
        }

        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            SizeToContent = SizeToContent.WidthAndHeight;

            if (File.Exists(_configFile))
            {
                DataContext = ScanConfig.Deserialize(_configFile);

                //PropertyChanged(this, new PropertyChangedEventArgs("Config"));
            }

            SetAppsEnabled();

            //var drives = DriveInfo.GetDrives()
            //    .Where(x => x.DriveType == DriveType.Fixed);


            //foreach (var drive in drives)
            //{
            //    var phpDirs = SearchForPHPDirs(drive.RootDirectory).ToArray();
            //    var folders = drive.RootDirectory.GetDirectories("*", SearchOption.TopDirectoryOnly);
            //}
        }

        void Window1_Closing(object sender, CancelEventArgs e)
        {
            SaveConfig();
        }

        private void ShowDialog(string Message, string Title)
        {
            new DialogWindow()
            {
                Message = Message,
                Owner = this,
                Title = Title,
                WindowStartupLocation = WindowStartupLocation.CenterOwner
            }.ShowDialog();
        }

        private bool Validate()
        {
            var messages = Config.Validate();

            if (!File.Exists(_phpvhExe))
                messages.Add("phpvh.exe not found");

            if (messages.Count == 0)
                return true;

            ShowDialog(messages.Aggregate((x, y) => x + "\r\n\r\n" + y),
                "Validation Errors");

            return false;
        }

        private void RunPhpvh(string args)
        {
            var process = new Process()
            {
                StartInfo = new ProcessStartInfo()
                {
                    Verb = "runas",
                    FileName = _phpvhExe,
                    Arguments = args,                    
                },
                
            };

            try
            {
                process.Start();
            }
            catch (Win32Exception ex)
            {
                MessageBox.Show("Error starting phpvh\r\n\r\n" + ex.Message,
                    "Error starting phpvh", MessageBoxButton.OK, MessageBoxImage.Error);

                return;
            }            

            Environment.Exit(0);
        }

        private void StartButton_Click(object sender, RoutedEventArgs e)
        {
            SaveConfig();

            if (!Validate())
                return;            

            RunPhpvh(Config.GetArgs() + " -l");
        }

        private void WebrootBrowseButton_Click(object sender, RoutedEventArgs e)
        {
            if (Directory.Exists(Config.WebRoot))
                _folderDialog.SelectedPath = Config.WebRoot;

            if (_folderDialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Config.WebRoot = _folderDialog.SelectedPath;                
            }
        }

        private void AppsButton_Click(object sender, RoutedEventArgs e)
        {

        }

        private void SetAppsEnabled()
        {
            AppsTextBox.IsEnabled = !Config.AllApps;
        }

        private void ScanAllCheckBox_Click(object sender, RoutedEventArgs e)
        {
            SetAppsEnabled();
        }

        private void ExitMenuItem_Click(object sender, RoutedEventArgs e)
        {
            SaveConfig();

            Environment.Exit(0);
        }

        private void HelpMenuItem_Click(object sender, RoutedEventArgs e)
        {
            new Guide().Show();
            //Process.Start("http://www.autosectools.com/Page/PHP-Vulnerability-Hunter-Guide");
        }

        private void RepairButton_Click(object sender, RoutedEventArgs e)
        {
            if (!Validate())
                return;

            var config2 = Config.Clone() as ScanConfig;
            config2.Repair = true;

            RunPhpvh(config2.GetArgs() + " -l");
        }
    }
}
